*-----------------------------------------------------------
* Title      : cabal -> cabal neo-geo mvs
* Written by : iq_132
* Date       : 2020
* Description:
*-----------------------------------------------------------
    ORG    $2d800
START:                  ; first instruction of program

    jmp     _init_clear_ram         ; 2d800
    rts     ; 8 byte align
    jmp     _copy_sprites           ; 2d808
    rts     ; 8 byte align
    jmp     _vblank_routine         ; 2d810
    rts     ; 8 byte align
    jmp     _copy_sprites           ; 2d818
    rts     ; 8 byte align
    jmp     _copy_background        ; 2d820
    rts     ; 8 byte align
    jmp     _copy_txt_layer         ; 2d828
    rts     ; 8 byte align
    jmp     _inputs_get_uldr        ; 2d830
    rts     ; 8 byte align
    jmp     _input_get_buttons      ; 2d838
    rts     ; 8 byte align
    jmp     _inputs_get_coin        ; 2d840
    rts     ; 8 byte align
    jmp     _coin_sound             ; 2d848
    rts     ; 8 byte align
    jmp     _write_soundlatch       ; 2d850
    rts     ; 8 byte align
    jmp     _get_dip_settings       ; 2d858
    rts



_write_soundlatch
    cmpi.b  #$ff, D0
    bne     _write_soundlatch_normal
    move.b  #$1f, D0    ; write $3f
_write_soundlatch_normal
    addi.b  #$20, D0
    move.b  D0, $320000
    rts

_coin_sound
    move.b  #$40, $320000
    rts


;----------------------------------------------------------------------------------
; Read dip settings
;    1. Read hard-coded dips (AES/CD)
;    2. Verify this is an AES/CD / MVS machine
;    3. Read dip menu settings (MVS only)
;    4. Read dip setting for free play (MVS only)
;----------------------------------------------------------------------------------
_get_dip_settings
	move.w  $2d7fe.l, D0	; hard-coded (AES/CD)
	not.w   D0
	cmpi.b  #$80, $10fd82
	bne	    _no_free_play   ; not MVS mode, no dip switches!
	clr.w	D0
	or.b	$10fd8f, D0	    ; demo sounds
	lsl.w   #$3, D0
	or.b	$10fd8e, D0	    ; difficulty (2 bits)
	lsl.w	#$2, D0
	or.b	$10fd8d, D0	    ; bonus life
	lsl.w	#$2, D0
	or.b	$10fd8c, D0	    ; lives (not working...)
	lsl.w	#$3, D0
	or.b	$10fd8b, D0	    ; invert buttons (swap)
	lsl.w	#$5, D0
	or.b	$10fd8a, D0     ; coinage settings
	btst	#$6, $300001    ; check free play dip setting
	bne	    _no_free_play
	ori.b	#$f, D0
_no_free_play
	move.w  D0, $10000c
	rts


;----------------------------------------------------------------------------------
; Read coins and make sure only read once
;----------------------------------------------------------------------------------
_inputs_get_coin
    cmpi.b  #$80, $10fd82
    beq     _coin_mvs           ; normal coin presses
	move  #$3, D0
	btst	#$1, D1
	bne	    _p1_select_not_pressed
	eor.w	#1, D0
_p1_select_not_pressed
	btst	#$3, D1
	bne     _p2_select_not_pressed
	eor.w	#$2, D0
_p2_select_not_pressed
    bra     _process_coins
_coin_mvs
	move.b	$320001, D0
_process_coins
	not.b	D0
	andi.b	#$3, D0
	move.b	$10cfff, D2 ; get previous coin
	move.b	D0, $10cfff ; save current coin
	cmpi.b	#$0, D0
	beq	    _NO_coin
    move.b   #$0, D1
    btst    #$0, D0
    beq     _no_coin1
    btst    #$0, D2
    bne     _no_coin1
    ori.b   #$1, D1
_no_coin1
    btst    #$1, D0
    beq     _no_coin2
    btst    #$1, D2
    bne     _no_coin2
    ori.b   #$2, D1
_no_coin2
    cmpi.b  #$0, D1
    beq     _NO_coin
    move.b  #$40, $320000
	ori.b	#$80, D1
	move.b  D1, D0
_NO_coin
	rts


;----------------------------------------------------------------------------------
; Get up/down/left/right directional inputs and put them in the correct format
;----------------------------------------------------------------------------------
_inputs_get_uldr
	move.b	$10fd96, D0 ;$300000, D0 ; right!
;	not.b	D0
	andi.w	#$f, D0
	move.b	$10fd9c, D1 ;$340000, D1
;	not.b	D1
	andi.w	#$f, D1
	move.w  D0, $10001e.l
	move.w  D1, $100020.l
	rts


;----------------------------------------------------------------------------------
; Get inputs and modify them to match what Cabal expects
;----------------------------------------------------------------------------------
_input_get_buttons
	move.w	#$ffff, D7
;	move.b  #$0, $380001    ; select input
;	move.b  D0, $380061     ; set input bank
	move.b	$10FD96, D0 ; $300000, D0
	not.b   D0
	btst	#$4, D0
	bne	_p1_button1_not_pressed
	eor.w	#$1, D7
_p1_button1_not_pressed
	btst	#$5, D0
	bne	_p1_button2_not_pressed
	eor.w	#$2, D7
_p1_button2_not_pressed
	btst	#$6, D0
	bne	_p1_button3_not_pressed
	eor.w	#$2000, D7
_p1_button3_not_pressed
	move.b	$10fd9c, D0 ;$340000, D0
	not.b   D0
	btst	#$4, D0
	bne	_p2_button1_not_pressed
	eor.w	#$4, D7
_p2_button1_not_pressed
	btst	#$5, D0
	bne	_p2_button2_not_pressed
	eor.w	#$8, D7
_p2_button2_not_pressed
	btst	#$6, D0
	bne	_p2_button3_not_pressed
	eor.w	#$1000, D7
_p2_button3_not_pressed
	move.b	$380000, D0
	btst	#$0, D0
	bne	_p1_start_not_pressed
	eor.w	#$8000, D7
_p1_start_not_pressed
	btst	#$2, D0
	bne	_p2_start_not_pressed
	eor.w	#$4000, D7
_p2_start_not_pressed
	rts


;----------------------------------------------------------------------------------
; Initilization functions
;   1. Clear main ram 40000-42fff
;   2. Clear main ram 100000-102fff
;   3. Copy Palette is pre-modified from xxxxbbbbggggrrrr to Neo-Geo format
;      xrgbrrrrggggbbbb in the rom patches.
;   4. Fill sprite zoom table with full sizes, Cabal has no zooming capabilities
;   5. Fill background layers with 'sticky' bits and set offsets to 0.
;      Cabal does not have any background scrolling capabilities
;----------------------------------------------------------------------------------
_init_clear_ram
    clr.l   D1
    move.b  D0, $300001.l       ; poke watchdog
    jsr     $C004D4             ; init controllers
    move.l  #$fff, D0           ; clear 100000-103fff
    lea     $100000.l, A1
_clear_mainram_loop
    move.l  D1, (A1)+
    dbra    D0, _clear_mainram_loop
    ; palette init
	move.l  #$30f, D0
	lea     $2cf70.l, A0    ; colors are here...
	lea     $400000.l, A1
_palette_init_loop
	move.w  (A0)+, (A1)+
	dbra    D0, _palette_init_loop
	move.w  #$8000, $400000	        ; set reference color
	move.w	$400400, $401ffe	; set background color...
	jsr     $C004C2 ; clear fix layer
	jsr     $c004c8 ; clear sprite SCB2-4, SCB1
    ; let's configure the background layer(s) here since the background position is static
    move.w  #$1, $3c0004
    move.w  #$1f, D1
    move.w  #$40, D0    ; sticky
    move.w  #$8200, $3c0000
_sprite_ypos_clear
    move.w  D0, $3c0002
    dbra    D1, _sprite_ypos_clear
    move.w  #$10,   $3c0004
    move.w  #$8200, $3c0000 ; set background y pos defaults
    move.w  #$0010, $3c0002
    move.w  #$0010, $3c0002
    move.w  #$8400, $3c0000 ; set background x pos defaults
    move.w  #$1000, $3c0002
    move.w  #$1000, $3c0002
    rts

;----------------------------------------------------------------------------------
; VBlank routine
;   1. Ack IRQ
;   2. The BIOS calls this before the init function, over-ride it to init properly
;   3. Call graphics copying routines
;----------------------------------------------------------------------------------
_vblank_routine
    cmpi.b  #$1, $10f2ff
    beq     _use_game_vblank
    move.b  #$1, $10f2ff
    jmp     $122        ; hijack the vblank routine early on and call our init
_use_game_vblank
    move.w  #$7, $3c000c.l  ; ack irq
_do_vblank
    jsr     $C0044A ; SYSTEM_IO
    move    #$2700, SR  ; disable interrupts
    move.b  D0, $300001.l
    jsr     $5ba.l              ; coin/sound...
    tst.w   $100044.l
    beq     _skip_background_copy
    jsr     _copy_background    ;   $756.l              ; copy tilemap ram
_skip_background_copy
    jsr     _input_get_buttons  ; call first! (inputs)
    jsr     $78E.l              ; process inputs...
    addq.w  #1, $100038.l
    jsr     _copy_txt_layer
    jsr     _copy_sprites
    movea.l $10003a.l, A0
    jsr     (A0)
    rte


;----------------------------------------------------------------------------------
; Text layer copying
; 1. Change offsets from (y * 32) + x -> ((x + 4) * 32) + y using look-up table
;    hard-coded in rom. Note that the +4 is due to Neo-Geo text layer be 40 tiles
;    wide vs. the more standard size of 32x32 like Cabal.
; 2. No modifications are necessary color data, we address the color depth change 
;    2bpp->4bpp by adding the color data to the pixel data on init!
; 3. Modify the code/tile number + 0x80 to add standard fonts
;----------------------------------------------------------------------------------
_copy_txt_layer
 	lea.l	$104000.l, A0       ; 60000
	lea.l	$2f000.l, A1
	move.l	#$3ff, D0
_txt_loop
	move.w	(A1)+, $3c0000
	move.w  (A0)+, D1
	move.w  D1, D2
	addi.w  #$80, D2
	andi.w  #$3ff, D2
	andi.w  #$fc00, D1
	or.w    D1, D2
	move.w	D2, $3c0002
	dbra	D0, _txt_loop
	rts


;----------------------------------------------------------------------------------
; Background layer copying
; 1. Change offsets from (y * 16) + x -> (x * 16) + y using look-up table
;    hard-coded in rom.
; 2. Modify tile number and color offset for use as Neo-Geo sprite
; 3. Make second layer behind this background layer to correct for Neo-Geo sprite 
;    pixel value of 0 always being transparent. Background layer should be opqaue!
;----------------------------------------------------------------------------------
_copy_background
    move.l  #$ff, D0
    lea     $103000.l, A0
    lea     $2f800.l, A1        ; LUT IS HERE
    move.w  #$1, $3c0004
_bg_copy_loop
    move.w  (A0)+, D2           ; get tile data
    move.w  (A1)+, D3           ; get offset 
    move.w  D3, $3c0000         ; SET OFFSET
    move.w  D2, D1              ; COPY TILE + COLOR
    andi.w  #$fff, D1           ; MAKE ONLY TILE
    addi.w  #$3b, D1
    move.w  D1, $3c0002         ; MOVE TILE INTO OFFSET
    andi.w  #$f000, D2          ; MAKE COLOR ONLY
    lsr.w   #$4, D2             ; SHIFT TO RIGHT = F00
    ori.w   #$2000, D2          ; COLOR += 200
    move.w  D2, $3c0002         ; MOVE COLOR TO OFFSET+1
    andi.w  #$3fe, D3           ; OFFSET &= 3fe - was | 400
    move.w  D3, $3c0000         ; SET OFFSET
    lsr.w   #$8, D2             ; COLOR TO RIGHT = F
    andi.w  #$f, D2             ; COLOR &= F
    ori.w   #$A90, D2           ; solid background fill is here!
    move.w  D2, $3c0002         ; move tile to offset
    move.w  #$3000,$3c0002      ; background color = 300
    dbra    D0, _bg_copy_loop
    move.w  #$0, $100044.l       ; clear force update byte
    rts


;----------------------------------------------------------------------------------
; Sprite copying
; 1. Modify sprite control data (x, y, flip, color, code) to Neo-Geo format
; 2. Adjust colors and offsets for sprites
;----------------------------------------------------------------------------------
_copy_sprites
	lea.l		$103800, A0
	move.l		#$ff, D7
_sprite_loop
	move.w		($0, A0), D1    ; this is sprite Y
	btst		#$8, D1
	beq			_skip_sprite    ; skip if (A0 & 0x100) == 0
	tst.b       D1
	beq         _skip_sprite    ; skip if y == 0
    andi.w      #$ff, D1    ; sprite y pos
    eor.w       #$1ff, D1
    lsl.w       #$7, D1
    ori.w       #$1, D1
    move.w      ($2, A0), D2    ; code
    andi.w      #$fff, D2
    addi.w      #$103b, D2  ; add + 0x103b sprites ('NEO-GEO' logo 0-3b, 3b-103b background tiles, 103b-203b sprites, 203b+ blank)
    move.w      ($4, A0), D3    ; sx / flipx / color
    move.w      D3, D4
    andi.w      #$7800, D4      ; color
    lsr.w       #$3, D4
    ori.w       #$1000, D4      ; color offset = 100
    btst        #$a, D3     ; set flipx, btst+beq+or faster than bit operations 14/40 vs 68 cycles
    beq         _no_flipx
    ori.w       #$1, D4
_no_flipx
    addi.w      #$20, D3        ; fix pos
    lsl.w       #$7, D3         ; sx
    move.w      D7, D6
    addi.w      #$8220, D6
    move.w      #$200, $3c0004
    move.w      D6, $3c0000
    move.w      D1, $3c0002 ; ypos
    move.w      D3, $3c0002 ; xpos
    move.w      #$1, $3c0004
    andi.w      #$1ff, D6
    lsl.w       #$6, D6
    move.w      D6, $3c0000
    move.w      D2, $3c0002 ; tile
    move.w      D4, $3c0002
    addq.w      #$8, A0
	dbra		D7, _sprite_loop
	rts
_skip_sprite
	move.w		D7, D0
	addi.w		#$8220, D0
	move.w		D0, $3c0000		; set address
	move.w		#$0, $3c0002		; set ypos/size/sticky value
    addq.w      #$8, A0
	dbra		D7, _sprite_loop
	rts

    END    START        ; last line of source
















*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
